/**
***     SuspendApp.c
***
***     Copyright (C) 2006 SophiaCradle Incorporated.
***     All rights reserved.
**/

#include "AEEModGen.h"
#include "AEEAppGen.h"
#include "AEEStdLib.h"
#include "AEEShell.h"
#include "Common.h"
#include "SuspendApp.bid"
#include "AEEWeb.h"

#define     ACCESS_URL      ("www.google.com")

// Avbg\
typedef struct SuspendApp {
    AEEApplet a; // Avbg\̂̐擪o͕K AEEApplet ^ɂ邱
    Common common; // ʕ`p

    // ʐMp
    IWeb* web;
    AEECallback callback;
    IWebResp* webresp;
    IGetLine* getline;
} SuspendApp;

static boolean SuspendApp_HandleEvent(SuspendApp* app, AEEEvent eCode, uint16 wParam, uint32 dwParam);
static boolean OnAppStart(SuspendApp* app);
static boolean OnAppStop(SuspendApp* app);
static boolean OnAppSuspend(SuspendApp* app);
static boolean OnAppResume(SuspendApp* app);
static boolean OnKey(SuspendApp* app, uint16 key);
static void WebAction_Start(SuspendApp* app, char* pszUrl);
static void WebAction_GotResp(void* p);
static void WebAction_Header(void* p, const char* name, GetLine* val);
static void WebAction_Status(void* p, WebStatus webstatus, void* val);
static void WebAction_Stop(SuspendApp* app);
static void WebAction_ReadLines(void* p);

int AEEClsCreateInstance(AEECLSID ClsId, IShell* pIShell, IModule* po, void** ppObj)
{
    *ppObj = NULL;
    if (ClsId == AEECLSID_SUSPENDAPP) {
        if (AEEApplet_New(sizeof(SuspendApp), ClsId, pIShell, po, (IApplet**)ppObj, (AEEHANDLER)SuspendApp_HandleEvent, NULL) == TRUE) {
            return AEE_SUCCESS;
        }
    }
    return EFAILED;
}

static boolean SuspendApp_HandleEvent(SuspendApp* app, AEEEvent eCode, uint16 wParam, uint32 dwParam)
{
    switch (eCode) {
        case EVT_APP_START:
            return OnAppStart(app);

        case EVT_APP_STOP:
            return OnAppStop(app);

        case EVT_APP_SUSPEND:
            return OnAppSuspend(app);

        case EVT_APP_RESUME:
            return OnAppResume(app);

        case EVT_KEY:
            return OnKey(app, wParam);
    }
    return FALSE;
}

// AvbgJnƂɌĂяoB
static boolean OnAppStart(SuspendApp* app)
{
    IShell* shell = app->a.m_pIShell;
    IDisplay* display = app->a.m_pIDisplay;
    WebOpt webopts[2];

    app->web = NULL;
    ISHELL_CreateInstance(shell, AEECLSID_WEB, (void**)&app->web);
    if (app->web != NULL) {
        // ڑ̃^CAEgԂ̐ݒ (ms)
        webopts[0].nId = WEBOPT_CONNECTTIMEOUT;
        webopts[0].pVal = (void*)10000;

        // WebOpt I}[N
        webopts[1].nId = WEBOPT_END;

        // IvVݒ
        IWEB_AddOpt(app->web, webopts);

        COMMON_Init(&app->common, display, shell, 32768);
        COMMON_WriteString(&app->common, "Push Select Key...\n");
        COMMON_Draw(&app->common);
        return TRUE;
    }
    return FALSE;
}

//EVT_APP_STOP Cxg̏
static boolean OnAppStop(SuspendApp* app)
{
    // eJiTXyhɓdɉȂ悤ɒӁj
    CALLBACK_Cancel(&app->callback);
    if (app->web != NULL) {
        IWEB_Release(app->web);
    }
    COMMON_Free(&app->common);
    return TRUE;
}

//EVT_APP_SUSPEND Cxg̏
static boolean OnAppSuspend(SuspendApp* app)
{
    // TXyhƂL^
    COMMON_WriteString(&app->common, "*** suspended!\n");
    // eJ
    CALLBACK_Cancel(&app->callback);
    IWEB_Release(app->web);
    app->web = NULL;
    return TRUE;
}

//EVT_APP_RESUME Cxg̏
static boolean OnAppResume(SuspendApp* app)
{
    IShell* shell = app->a.m_pIShell;

    // W[ƂL^
    COMMON_WriteString(&app->common, "*** resumed!\n");
    // eCX^X쐬
    app->web = NULL;
    ISHELL_CreateInstance(shell, AEECLSID_WEB, (void**)&app->web);
    if (app->web != NULL) {
        // ʂ̍ĕ`
        COMMON_Draw(&app->common);
        return TRUE;
    }
    return FALSE;
}

// L[ꂽƂɌĂяoB
static boolean OnKey(SuspendApp* app, uint16 key)
{
    switch (key) {
        case AVK_SELECT:
            // ʐMJn
            WebAction_Start(app, "http://www.google.com");
            return TRUE;

        case AVK_DOWN:
            COMMON_NextLine(&app->common);
            COMMON_Draw(&app->common);
            return TRUE;

        case AVK_UP:
            COMMON_PreviousLine(&app->common);
            COMMON_Draw(&app->common);
            return TRUE;
    }
    return FALSE;
}

// ʐMJn
static void WebAction_Start(SuspendApp* app, char* url)
{
    CALLBACK_Init(&app->callback, WebAction_GotResp, app);
    IWEB_GetResponse(app->web,
                    (app->web, &app->webresp, &app->callback, url,
                    WEBOPT_HANDLERDATA, app,
                    WEBOPT_HEADER, "X-Method: GET\r\n",
                    WEBOPT_HEADERHANDLER, WebAction_Header,
                    WEBOPT_STATUSHANDLER, WebAction_Status,
                    WEBOPT_END));
    return;
}

// bZ[W̎擾Jn
static void WebAction_GotResp(void* p)
{
    SuspendApp* app = (SuspendApp*)p;
    WebRespInfo* info;
    char buffer[80];

    info = IWEBRESP_GetInfo(app->webresp);
    SPRINTF(buffer, "** got response...\n** info code: %d\n", info->nCode);
    COMMON_WriteString(&app->common, buffer);
    COMMON_Draw(&app->common);
    if (info->pisMessage != NULL) {
        ISourceUtil* util = NULL;

        ISHELL_CreateInstance(app->a.m_pIShell, AEECLSID_SOURCEUTIL, (void**)&util);
        ISOURCEUTIL_GetLineFromSource(util, info->pisMessage, 1100, &app->getline);
        ISOURCEUTIL_Release(util);
    }
    if (app->getline != NULL) {
        WebAction_ReadLines(app);
    }
    else {
        COMMON_WriteString(&app->common, "** no response\n");
        COMMON_Draw(&app->common);
        WebAction_Stop(app);
    }
    return;
}

// wb_[
static void WebAction_Header(void* p, const char* name, GetLine* val)
{
    SuspendApp* app = (SuspendApp*)p;

    if (name != NULL) {
        COMMON_WriteString(&app->common, name);
        COMMON_WriteString(&app->common, ":");
        COMMON_WriteString(&app->common, val->psz);
        COMMON_WriteString(&app->common, "\n");
    }
    else {
        COMMON_WriteString(&app->common, val->psz);
        COMMON_WriteString(&app->common, "\n");
    }
    COMMON_Draw(&app->common);
    return;
}

// Ԏ擾
static void WebAction_Status(void* p, WebStatus webstatus, void* val)
{
    char* string = NULL;
    SuspendApp* app = (SuspendApp*)p;
    (void)val;

    switch (webstatus) {
        case WEBS_CANCELLED:
            string = "** cancelled...\n";
            break;

        case WEBS_GETHOSTBYNAME:
            string = "** finding host...\n";
            break;

        case WEBS_CONNECT:
            string = "** connecting...\n";
            break;

        case WEBS_SENDREQUEST:
            string = "** sending...\n";
            break;

        case WEBS_READRESPONSE:
            string = "** receiving...\n";
            break;

        case WEBS_GOTREDIRECT:
            string = "** redirect...\n";
            break;

        case WEBS_CACHEHIT:
            string = "** cache hit...\n";
            break;
    }
    if (string != NULL) {
        COMMON_WriteString(&app->common,string);
        COMMON_Draw(&app->common);
    }
    return;
}

// ʐMI
static void WebAction_Stop(SuspendApp* app)
{
    CALLBACK_Cancel(&app->callback);
    if (app->getline != NULL) {
       IGETLINE_Release(app->getline);
       app->getline = NULL;
    }
    if (app->webresp != NULL) {
       IWEBRESP_Release(app->webresp);
       app->webresp = NULL;
    }
    return;
}

// bZ[W擾
static void WebAction_ReadLines(void* p)
{
    int value;
    GetLine getline;
    SuspendApp* app = (SuspendApp*)p;

    value = IGETLINE_GetLine(app->getline, &getline, IGETLINE_LF);
    if (value == IGETLINE_WAIT) {
        CALLBACK_Init(&app->callback, WebAction_ReadLines, app);
        IGETLINE_Peekable(app->getline, &app->callback);
    }
    else {
        COMMON_WriteString(&app->common, getline.psz);
        COMMON_WriteString(&app->common, "\n");
        if (!IGETLINE_Exhausted(value)) {
            CALLBACK_Init(&app->callback, WebAction_ReadLines, app);
            IGETLINE_Readable(app->getline, &app->callback);
        }
        else {
            COMMON_WriteString(&app->common, (value == IGETLINE_ERROR) ? ("\n** error\n") : ("\n** end\n"));
            COMMON_Draw(&app->common);
            WebAction_Stop(app);
        }
    }
    return;
}
